@@ -1,8 +1,12 @@
-class BaseDimension(object):
+from abc import ABC, abstractmethod
+
+
+class BaseDimension(ABC):
     """
     Base class for the space definition of data types
     """
 
+    @abstractmethod
     def sample(self):
         """
         Sample a random value from the assigned distribution
@@ -10,4 +14,4 @@ class BaseDimension(object):
 
         raise NotImplementedError(
             "The sample method must be defined according each data type handler"
-        )
+        )  # pragma: no cover
